@@ -36,7 +36,7 @@ sub config {
 }
 
 package # hide from PAUSE
-    DBIx::DBO::DBD::mysql::Common;
+    DBIx::DBO::Common::DBD::mysql;
 use DBIx::DBO::Common;
 
 sub _build_sql_select {
@@ -47,7 +47,22 @@ sub _build_sql_select {
 }
 
 package # hide from PAUSE
-    DBIx::DBO::DBD::mysql::Query;
+    DBIx::DBO::Table::DBD::mysql;
+use DBIx::DBO::Common;
+
+sub _save_last_insert_id {
+    my $me = shift;
+    my $sth = shift;
+    return $sth->{mysql_insertid};
+}
+
+sub last_insert_id {
+    my $me = shift;
+    $me->{LastInsertID};
+}
+
+package # hide from PAUSE
+    DBIx::DBO::Query::DBD::mysql;
 use DBIx::DBO::Common;
 
 sub found_rows {
